#!/usr/bin/python
# -*- coding: UTF-8 -*-
import logging.config


# _formatter = logging.Formatter('logging_time: %(asctime)s - logging_level: %(levelname)s - logging_message: %(message)s'
#                                '- program_pid: %(process)d')
#
# main_logger = logging.getLogger('main_logger')
#
# # 设置全局级别
# main_logger.setLevel(logging.INFO)
#
# # 创建控制台处理器
# console_handler = logging.StreamHandler(sys.stdout)
#
# # 设置控制台等级
# console_handler.setLevel(logging.INFO)
#
# # 给处理器设置输出格式
# console_handler.setFormatter(_formatter)
#
# # 日志器添加处理器
# main_logger.addHandler(console_handler)

config = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "simple": {
            "format": "logging_time: %(asctime)s - logging_level: %(levelname)s - logging_message: %(message)s "
                      "- program_pid: %(process)d"
        }
    },
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
            "level": "DEBUG",
            "formatter": "simple",
            "stream": "ext://sys.stdout"
        },
    },
    "loggers": {
        "main_logger": {
            "level": "INFO",
            "handlers": ["console"]
        }
    }
}

logging.config.dictConfig(config)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import time
import pika
import traceback
from lib.config import setting
from pika.exceptions import ConnectionClosed, ChannelClosed

amqp_conf = setting['base_amqp']  # type: dict


class MQConnection(object):
    def __init__(self, kwargs):
        self.kwargs = kwargs
        self.connection = None
        self.channel = None

    def reconnect(self):
        if self.connection and not self.connection.is_closed:
            self.connection.close()
        credentials = pika.PlainCredentials(self.kwargs.get('username', 'guest'), self.kwargs.get('passwd', 'guest'))
        parameters = pika.ConnectionParameters(host=self.kwargs.get('hostname', 'localhost'),
                                               port=int(self.kwargs.get('port', 5672)),
                                               virtual_host=self.kwargs.get('virtual_host', '/'),
                                               credentials=credentials,
                                               connection_attempts=int(self.kwargs.get('connection_attempts', 3)),
                                               retry_delay=int(self.kwargs.get('retry_delay', 10)),
                                               blocked_connection_timeout=int(self.kwargs.get('blocked_connection_timeout', 5)),
                                               heartbeat=int(self.kwargs.get('heartbeat', 300))
                                               )
        self.connection = pika.BlockingConnection(parameters)
        self.channel = self.connection.channel()

        if isinstance(self, MQConsume):
            # self.channel.basic_qos(prefetch_count=10)
            # self.channel.basic_consume(queue=self.tbname, on_message_callback=self.consumer_callback, auto_ack=False)
            self.channel.basic_consume(queue=self.queue, on_message_callback=self.consumer_callback, auto_ack=True)
            self.channel.start_consuming()


class MQPublish(MQConnection):
    def __init__(self, kwargs):
        super(MQPublish, self).__init__(kwargs)
        self.reconnect()

    def _to_queue(self, ex, key, data, serializer=None, count=3):
        for i in range(count):
            try:
                if serializer:
                    data = serializer(data)
                self.channel.basic_publish(exchange=ex, routing_key=key, body=data,
                                           properties=pika.BasicProperties(delivery_mode=2))
                return True
            except pika.exceptions.StreamLostError:
                print('mq heartbeat publish error')
                self.reconnect()
                time.sleep(3)
            except Exception as e:
                traceback.print_exc()
                self.reconnect()
                time.sleep(3)
        return False


class MQConsume(MQConnection):
    def __init__(self, kwargs):
        super(MQConsume, self).__init__(kwargs)

    def consumer_callback(self, ch, method, header, body):
        # delivery_tag = method.delivery_tag

        # 开启子线程，确保消息被ack
        # t = threading.Thread(target=self.do_ack, args=(self.connection, ch, delivery_tag))
        # t.start()

        self._do(data=body)
    # def do_ack(self, conn, ch, delivery_tag):
    #     cb = functools.partial(self.ack_message, ch, delivery_tag)
    #     conn.add_callback_threadsafe(cb)

    # def ack_message(self, ch, deliver_tag):
    #     if ch.is_open:
    #         ch.basic_ack(deliver_tag)
    #     else:
    #         channel已经关闭，重连确认
    #         self.reconnect()
    #         self.channel.basic_ack(deliver_tag)

    def _do(self, data):
        # 真正消费的函数
        pass

    def start_consume(self, queue, count=3):
        self.queue = queue
        for i in range(count):
            try:
                self.reconnect()
            except ConnectionClosed as e:
                traceback.print_exc()
                time.sleep(3)
            except ChannelClosed as e:
                traceback.print_exc()
                time.sleep(3)
            except pika.exceptions.StreamLostError:
                print('mq heart beat receive error')
                time.sleep(3)
            except Exception as e:
                traceback.print_exc()
                time.sleep(3)

    def _from_queue(self, qname, count=3):
        # 消费入口函数
        self.start_consume(qname, count)


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import pymongo


def init_mongo(conf):
    """ 获取mongo连接实例 """
    kw = {
        'maxPoolSize': 10,  # 可选，设置mongo连接时的连接池
        'readPreference': 'primaryPreferred',
        'sale': False  # 连接不用ack，提升查询效率
    }
    addr_list = list(addr.strip() for addr in conf['addr'].split(','))
    if conf.get('mongodb_type') == 'repl':
        kw['replicaSet'] = conf['repl_name']
        conn = pymongo.MongoClient('mongodb://' + ','.join(map(lambda s: s.lstrip('mongodb://'), addr_list)), **kw)
    else:
        conn = pymongo.MongoClient(addr_list[0], **kw)
    # todo 钉钉通知 mongo 连接失败
    return conn



#!/usr/bin/python
# -*- coding: UTF-8 -*-
# requests 中文文档 https://docs.python-requests.org/zh_CN/latest/user/advanced.html#blocking-or-nonblocking

import requests
import time
from requests import adapters

requests.adapters.DEFAULT_RETRIES = 3  # 默认超时重连次数, requests可以对同一个host保持长连接
requests.adapters.DEFAULT_POOLSIZE = 30  # 默认连接池大小（协程数尽量不要超过此大小）
requests.adapters.DEFAULT_POOLBLOCK = True  # 当超过限制时，由pool_block参数决定是否阻塞等待。


def shopex_request(url, req_type='post', data='', **kwargs):
    try:
        if req_type == 'post':
            if type(data) == str:  # json格式提交请求
                r = requests.post(url, json=data, **kwargs)
            if type(data) == dict:  # 表单格式提交请求
                r = requests.post(url, data=data, **kwargs)
        elif req_type == 'get':
            r = requests.get(url, **kwargs)
        else:
            r = requests.post(url, data=data)
        r.raise_for_status()
        return 'succ', r.text
    except requests.exceptions.HTTPError as e:
        write_log("{}_{}".format(url, str(e)), method=kwargs.get('sys_log_tag', 'matrix'))
        if r.status_code >= 500:
            return 'fail', str(e)
        return 'dams', str(e)
    except requests.exceptions.ConnectTimeout as e:
        write_log("{}_{}".format(url, str(e)), method=kwargs.get('sys_log_tag', 'matrix'))
        return 'fail', str(e)
    except Exception as e:
        write_log("{}_{}".format(url, str(e)), method=kwargs.get('sys_log_tag', 'matrix'))
        return 'fail', str(e)


def write_log(msg, method):
    import syslog
    syslog.openlog(method, syslog.LOG_LOCAL0)
    syslog.syslog(syslog.LOG_INFO, msg)


def str2sec(src_str, format='%Y-%m-%d %H:%M:%S', float_num=False):
    try:
        time_tuple = time.strptime(src_str, format)
        timestamp = time.mktime(time_tuple)
        return float_num and timestamp or int(timestamp)
    except Exception as e:
        return ''


def sec2str(format='%Y-%m-%d %H:%M:%S', sec=None, gmtime=False):
    try:
        func = time.gmtime if gmtime else time.localtime
        return time.strftime(format, func(sec))
    except Exception as e:
        return ''
